#pragma once

typedef struct{
	char dev_name[256];
	char dev_path[1024];
}DEV_NAME;

typedef struct{
	int width;
	int height;
	float fps;
	int fps_num,fps_den;
	unsigned int codec;
	char codec_name[64];
}DEV_CAPACITY;

#define  MSG_ID_YUV420		0x01
#define  MSG_ID_PCMS16		0x02
#define  MSG_ID_RECORDERROR 0x03
/*
MSGID				PARAM				BUFFER				  BYTES				REMARK
MSG_ID_YUV420		0					pointer of buffer     size of buffer	return device video YUV420 data
MSG_ID_PCM16		0					pointer of buffer     size of buffer    return device audio s16 48KHZ 2ch data.
MSG_ID_RECORDERROR  error no.			error message buffer  0		
error code and mean:
1	Video encode is too slow, please reduce resolution and frame rate.
2	Copy video frame error.
3	No enough memory.
4	Video encode is too slow, queue is full.
5	No enough free space in disk.
6	Encode frame error.
7   Capture video frame too slow.
*/
typedef int (*CA_MESSAGE_CALLBACK)(int msgid,uint64_t param, void *buffer, int bytes);

#ifdef __cplusplus
extern "C" {
#endif

/*
[Parameters]:
	handle:[out] return api handle.
	dgb_log_path: [in] Specify the log file path name, can be NULL.
	productname: [in] specify your product name.
[Return]:
  1=OK, 0=failed.
[Remark]:
  
*/

int CA_Initalize(void**handle,const char* dbg_log_path,const char*productname);
/*
[Parameters]:
	handle:[in] api handle address.
[Return]:
  1=OK, 0=failed.
[Remark]:
  Release all and set handle to zero.
*/
void CA_Release(void **handle);

/*
[Parameters]:
	handle:[in] api handle.
	pdevs: [out] video capture device names.
	numofdevs: [in,out] specify size of pdevs.
	adevs: [out] audio capture device names.
	numofadevs: [in,out] specify size of adevs.
[Return]:
  1=OK, 0=failed.
[Remark]:
  Hw: Represents the original device. plughw: Indicates the modified software device
*/
int CA_GetCaptureDevices(void*handle, DEV_NAME *pdevs, int *numofdevs, DEV_NAME *adevs, int *numofadevs);

/*
[Parameters]:
	handle:[in] api handle.
	dev: [in] device name.
	cap: [out] device capacity.
	numofcap: [in,out] specify size of cap.
	curr: [out] specify current capacity.
[Return]:
  1=OK, 0=failed.
[Remark]:
*/
int CA_GetCaptureDeviceCapacity(void *handle,DEV_NAME *dev, DEV_CAPACITY *cap, int *numofcap, DEV_CAPACITY *curr);

/*
[Parameters]:
	handle:[in] api handle.
	dev: [in] device name.
	cap: [out] device capacity.
	numofcap: [in,out] specify size of cap.
	curr: [out] specify current capacity.
[Return]:
  1=OK, 0=failed.
[Remark]:
*/
int CA_GetCurrentDeviceCapacity(void*handle,DEV_CAPACITY *cap);

/*
[Parameters]:
	handle:[in] api handle.
	hVideo:[in] window to attach. see SDL_CreateWindowFrom() of SDL2 document. when hVideo =1 will popup a internal window.
				when hVideo =0 will not create render and audio render window, you can use this for a recording in background.
	bkeepaspect: [in] keep picture aspect.
	v_dev: [in] video device
	v_cap: [in] video capacity.
	a_dev: [in] audio device.
	msg_call: [in] notify caller some event and HW video&audio data. 
[Return]:
  1=OK, 0=failed.
[Remark]:
*/
int CA_OpenDevice(void*handle,unsigned long long hVideo,int bkeepaspect,DEV_NAME *v_dev,DEV_CAPACITY *v_cap,DEV_NAME *a_dev,CA_MESSAGE_CALLBACK msg_call=NULL);

/*
[Parameters]:
	handle:[in] api handle.
Return]:
  1=OK, 0=failed.
[Remark]:
*/
int CA_CloseDevice(void*handle);

/*
[Parameters]:
	handle:[in] api handle.
	w,h:[in] video render window
Return]:
  1=OK, 0=failed.
[Remark]:
*/
void CA_UpdateVideoSize(void*handle,int w, int h);

/*
[Parameters]:
	handle:[in] api handle.
	v: [in] -255: mute. 255:unmute. 0~100: volume
Return]:
  1=OK, 0=failed.
[Remark]:
*/
void CA_AdjustVolume(void*handle,int v);
 
/*
[Parameters]:
	handle:[in] api handle.
	mp4path: [in] mp4 saving path
	w: [in] width for picture to recording. supported scale.
	h: [in] height for picture
	fps: [in] fps (max 60) for recording 
	bitrate: [in] bitrate for recording
	speed: [in] (value 0~3) fast speed is 0. best picture quality is 3
	format: [in] file format mp4
Return]:
  1=OK, 0=failed.
[Remark]:
*/
int CA_StartRecord(void *handle,const char* mp4path, int w, int h, int fps, int bitrate,int speed=0,const char*format="mp4");

/*
[Parameters]:
	handle:[in] api handle.
	bpause:[in] pause recording
Return]:
  1=OK, 0=failed.
[Remark]:
*/
int CA_PauseRecord(void *handle,int bpause);

/*
[Parameters]:
	handle:[in] api handle.
[Return]:
  1=OK, 0=failed.
[Remark]:
*/
int CA_StopRecord(void *handle);

/*
[Parameters]:
	handle:[in] api handle.
	fpath: [in] bmp file path.
Return]:
  1=OK, 0=failed.
[Remark]:
 only support bmp shot cut.
*/
int CA_ScreenShot_bmp(void* handle, const char* fpath);

/*
[Parameters]:
	handle:[in] api handle.
Return]:
  1=OK, 0=failed.
[Remark]:
*/
int CA_CheckIsRegistered(void *handle);

/*
[Parameters]:
	handle:[in] api handle.
Return]:
  1=OK, 0=failed.
[Remark]:
*/
int CA_Register(void *handle, const char*code, const char *name, const char* email);

/*
[Parameters]:
	handle:[in] api handle.
Return]:
  1=OK, 0=failed.
[Remark]:
*/
int CA_ClearRegisterCode(void *handle);

/*
[Parameters]:
	handle:[in] api handle.
	ispreview: [out] is opened device? can be null.
	ispictshow: [out] is render play? can be null.
	isrecord: [out] is recording? can be null.
	isgif: no use, set to NULL.
Return]:
  1=OK, 0=failed.
[Remark]:
*/
int CA_CheckState(void *handle,int *ispreview,int *ispictshow, int *isrecord, int *isgif);


const char* CA_GetDebugInfo(void *handle);
void CA_Log(const char *fmt, ...);

#ifdef __cplusplus
}
#endif