#include "mainwindow.h"
#include "ui_mainwindow.h"
#include "../SDK/include/ACapture.h"
#include <qfiledialog.h>
#include <QStandardPaths>
#include <QMessageBox>
#include <QTimer>
#include "registerdialog.h"

static void* api = nullptr;
static DEV_NAME g_devs[50];
static int              g_nb_devs=0;
static DEV_NAME g_adevs[50];
static int              g_nb_adevs=0;
static DEV_CAPACITY g_cap[100];
static int                     g_nb_cap=0;
static int                     g_cap_paused=0;
static int                     g_cap_err_code=0;
static char                  g_cap_err_str[1024];
static time_t                g_start_time = 0;

MainWindow::MainWindow(QWidget *parent)
    : QMainWindow(parent)
    , ui(new Ui::MainWindow)
{
    ui->setupUi(this);

    CA_Initalize(&api,"./ca-log.txt","prod");
    if (api==nullptr)
    {
        fprintf(stderr,"CA_Initalize failed.\n");
    }

    int ret;
    g_nb_devs = 50;
    g_nb_adevs = 50;
    ret = CA_GetCaptureDevices(api, g_devs, &g_nb_devs, g_adevs, &g_nb_adevs);
    if (ret)
    {
        //list video and audio device--------------------
        int i;
        for ( i = 0; i < g_nb_devs; i++)
        {
            ui->d_vDevice->addItem(QString::fromUtf8(g_devs[i].dev_name));
        }
        if (i>0) {
            ui->d_vDevice->setCurrentIndex(0);
        }
        for (i = 0; i < g_nb_adevs; i++)
        {
            ui->d_aDevice->addItem(QString::fromUtf8(g_adevs[i].dev_path));
        }
        if (i>0)
            ui->d_aDevice->setCurrentIndex(0);

    }

    ui->e_w->setText("1920");
    ui->e_h->setText("1080");
    ui->e_fps->setText("30");
    ui->e_bitratem->setText("10");
    ui->e_speed->setText("0");
    ui->d_slider->setValue(100);

   QTimer* time_clock = new QTimer(this);
   connect(time_clock,SIGNAL(timeout()),this,SLOT(timer1()));
   time_clock->start(1000);

   setWindowTitle("CloneAlliance Capture SDK demo 1.0");
}

MainWindow::~MainWindow()
{
    delete ui;
    CA_Release(&api);
}

int  ca_message_callback(int msgid,uint64_t param, void *buffer, int bytes)
{
    //NOTE: this is runing in a thread .
    if (msgid == MSG_ID_RECORDERROR)
    {
            g_cap_err_code = (int)param;
            strncpy(g_cap_err_str,(char*)buffer,1024);
    }
    else if (msgid == MSG_ID_PCMS16)
    {
         //audio data, you can modify it
    }
    else if (msgid == MSG_ID_YUV420)
    {
         //video data, you can modify it
    }

    return  0;
}

void MainWindow::timer1()
{
    if (g_start_time ==0)
        return;

    if (g_cap_paused)
        ui->rec_tip->setText("Recording paused...");
    else
    {
        int dur =(int)(time(nullptr) - g_start_time);
        QString tip = QString::asprintf("Recording %02d:%02d:%02d",dur/3600,dur%3600/60,dur%60);
        ui->rec_tip->setText(tip);
    }
    const char*info = CA_GetDebugInfo(api);
    if (info)
    {
        printf("%s\n",info);
    }
    if (g_cap_err_code)
    {
         g_start_time = 0;
         on_c_btnStop_clicked();
         QMessageBox::warning(this,"demo",QString::fromLatin1(g_cap_err_str));
    }
}

void MainWindow::on_d_btnOpen_clicked()
{
    int isopen,isrender,isrecord;
     if (CA_CheckState(api,&isopen,&isrender,&isrecord,0))
     {
            if (isopen)
            {
                  QMessageBox::warning(this,"demo","Already opened  the  device");
                  return;
            }
     }
    unsigned long long hVideo = ui->player->winId();
    int bkeepaspect = 1;
    DEV_NAME *v_dev = &g_devs[ui->d_vDevice->currentIndex()];
    DEV_CAPACITY *v_cap = &g_cap[ui->d_vCap->currentIndex()];
    DEV_NAME *a_dev = &g_adevs[ui->d_aDevice->currentIndex()];
   int ret =CA_OpenDevice(api,hVideo,bkeepaspect,v_dev,v_cap,a_dev,ca_message_callback);
   if (!ret)
       fprintf(stderr,"CA_OpenDevice failed.\n");
   else{
       QSize size = ui->player->size();
       CA_UpdateVideoSize(api,size.width(),size.height());
   }
}

void MainWindow::on_d_btnClose_clicked()
{
    int ret = CA_CloseDevice(api);
    if (!ret)
        fprintf(stderr,"CA_CloseDevice failed.\n");
}

void MainWindow::on_d_btnMute_clicked()
{
     static int bmute=0;
     bmute = !bmute;
     CA_AdjustVolume(api,bmute?-255:255);
     ui->d_btnMute->setText(bmute?"Unmute":"Mute");

}

void MainWindow::on_d_slider_valueChanged(int value)
{
      CA_AdjustVolume(api,value);
}

void MainWindow::on_c_btnStart_clicked()
{
    int isopen,isrender,isrecord;
     if (CA_CheckState(api,&isopen,&isrender,&isrecord,0))
     {
            if (!isopen)
            {
                  QMessageBox::warning(this,"demo","Please open the device first");
                  return;
            }
            if (isrecord)
            {
                  QMessageBox::warning(this,"demo","Please stop recording first");
                  return;
            }
     }

     int w =ui->e_w->text().toInt();
     int h = ui->e_h->text().toInt();
     int fps = ui->e_fps->text().toInt();
     int bitrateM = ui->e_bitratem->text().toInt();
     int speed = ui->e_speed->text().toInt();

     if (speed >3)
     {
        QMessageBox::warning(this,"demo","Capture speed only support 0~3 (0 mean max speed)");
        return;
     }


    QString mp4file;
     QString defdir =  QStandardPaths::writableLocation(QStandardPaths::DocumentsLocation);
    if (GetSaveFile(mp4file,"Mp4 file(*.mp4)",defdir))
    {
        if (mp4file.right(3) != "mp4")
            mp4file +=".mp4";
        int ret = CA_StartRecord(api,mp4file.toLatin1().data(),w,h,fps,bitrateM*1000*1000,speed);
        if (ret)
        {
              g_start_time = time(nullptr);
              g_cap_paused = 0;
              ui->c_btnStart->setEnabled(false);
        }
        else
             QMessageBox::warning(this,"demo","Start capture failed.");
    }
}

void MainWindow::on_c_btnPause_clicked()
{
       g_cap_paused = !g_cap_paused;
        CA_PauseRecord(api,g_cap_paused);

}

void MainWindow::on_c_btnStop_clicked()
{
    ui->c_btnStart->setEnabled(true);
    ui->rec_tip->setText("");
    g_start_time =0;
    CA_StopRecord(api);
}

void MainWindow::on_c_btnShot_clicked()
{
     QString bmpfile ;
     QString defdir =  QStandardPaths::writableLocation(QStandardPaths::DocumentsLocation);
    if (GetSaveFile(bmpfile,"BMP file(*.bmp)",defdir))
    {
         if (CA_ScreenShot_bmp(api,bmpfile.toLatin1().data()))
         {
              QMessageBox::information(this,"demo","Capture bmp OK.");
         }
    }
}

void MainWindow::on_c_btnReg_clicked()
{
      RegisterDialog reg(this);
      if (reg.exec() == QDialog::Accepted)
      {
           int ret = CA_Register(api, reg.m_code.toUtf8().data(), reg.m_name.toUtf8().data(), reg.m_email.toUtf8().data());
           if (ret)
           {
               QMessageBox::information(this,"demo","Register OK.");
           }
           else {
               QMessageBox::warning(this,"demo","Register failed.");
           }
      }
}

void MainWindow::on_c_btnUnreg_clicked()
{
     if (CA_ClearRegisterCode(api))
     {
         QMessageBox::information(this,"demo","Clear register key OK.");
     }
}

void MainWindow::on_d_vDevice_currentIndexChanged(int index)
{
     if (index <0) return;
     DEV_NAME *pcurdev = &g_devs[index];
     g_nb_cap = 100;
     DEV_CAPACITY curr;
     int ret = CA_GetCaptureDeviceCapacity(api,pcurdev, g_cap,&g_nb_cap, &curr);
     if (ret)
     {
         for (int i = 0; i < g_nb_cap; i++)
         {
             QString name = QString::asprintf("%dx%d@%dfps [%s]",g_cap[i].width,g_cap[i].height,(int)g_cap[i].fps,g_cap[i].codec_name);
             ui->d_vCap->addItem(name);
             if (g_cap[i].width == curr.width && g_cap[i].height == curr.height &&(int)g_cap[i].fps ==(int)curr.fps && g_cap[i].codec == curr.codec)
             {
                 ui->d_vCap->setCurrentText(name);
             }
         }
     }
     else
         fprintf(stderr,"CA_GetCaptureDeviceCapacity failed.\n");
}


bool   MainWindow::GetSaveFile(QString  &file,QString filter, QString defdir)
{
    QString fileName;
    fileName = QFileDialog::getSaveFileName(this, tr("Open"), "", filter);
   if (fileName.isEmpty())
       return  false;

   file = fileName;
   return true;
        /*
    QFileDialog   fd;
    fd.setWindowTitle("Save");
    fd.setAcceptMode(QFileDialog::AcceptOpen);
    fd.setDirectory(defdir);
     fd.setNameFilter(filter);
     fd.setFileMode(QFileDialog::ExistingFile);
     fd.setViewMode(QFileDialog::Detail);
    if(fd.exec() == QDialog::Accepted)
    {
       file = fd.selectedFiles()[0];
        return true;
    }
    return false;
 */
}
